#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 01/30/2000
#################################################################################
#
# Realm - Script part of Webchat designed to use the realms
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2R 0K9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement


use Webchat::Comm;						# Communication handler
use Webchat::Database;						# Database handler
use Webchat::Dbm; 						# DBM Addition
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table handler


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash


#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		TargetHandler($Database, \%param);
		Webchat::Comm::DataWrite($Database, \%param);
		DisplayHandler($Database, $cgi, \%param);
	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

####################
# Display Handler
#
# Display prompt for a private message, this includes a list of all the puppets
# et cetera. 

sub DisplayHandler						# Database, Cgi, Param
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# Cgi handle
	my $param = shift;					# Parameter handle

	my $who;						# List of puppets
	my $char;						# List of characters
	my $color;						# Colour choices
	my $size;						# Text colour

	my $defsize;						# Default size
	my $defcolor;						# Default colour
	my $delay;						# Delay of time

	my @who;						# List of puppets
	my @char;						# List of characters
	my @color;						# Colour choices
	my @size;						# Text colour

	my %chat;						# Chat hash
	my %realm;						# Realm DBM


	####################
	# Program area

	# Link DBM with hash
	Webchat::Dbm::DBMOpenChat(\%chat);
	Webchat::Dbm::DBMOpenRealm(\%realm);

	# Calculate delay
	$delay = (time - $realm{'SetTimeout'});

	# Unmangle data
	$param->{'USER'} = Webchat::Login::Unmangle($param->{'USER'});
	$param->{'CHAR'} = Webchat::Login::Unmangle($param->{'CHAR'});
	$param->{'ROOM'} = Webchat::Login::Unmangle($param->{'ROOM'});


	# Populate arrays
	OptionSplit(\@color, $chat{'OptTextColour'});
	OptionSplit(\@size,  $chat{'OptTextSize'});
	Webchat::Database::GetListPuppetYours($Database, \@char, $param->{'USER'});
	Webchat::Database::GetListWho($Database, \@who, $param->{'ROOM'}, $delay);

	# Retreive size and color
	($defsize, $defcolor) = Webchat::Database::DataGetFont($Database, $param->{'CHAR'}, $color[0]);

	# Format buttons
	$who    = $cgi->scrolling_list('TARGET', \@who, $who[0], 5);
	$char	= $cgi->popup_menu('CHAR',  \@char,  $param->{'CHAR'});
	$color	= $cgi->popup_menu('COLOR', \@color, $defcolor);
	$size   = $cgi->popup_menu('SIZE',  \@size,  $defsize);


	# HTML
	# Top and title
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $chat{'SetTop'});

	# Prompt and space
	Webchat::Table::MakeSingle($cgi, '<TEXTAREA NAME="POST", ROWS="3", COLS="55", WRAP="SOFT" ></TEXTAREA>');
	Webchat::Table::MakeBlank($cgi);

	# Prompt for From, To, Colour, Puppet
	Webchat::Table::MakeValid($cgi, $chat{'TagPuppet'},     $char);
	Webchat::Table::MakeValid($cgi, $chat{'TagPuppetTo'},   $who);
	Webchat::Table::MakeValid($cgi, $chat{'TagTextColour'}, $color);
	Webchat::Table::MakeValid($cgi, $chat{'TagTextSize'},   $size);
	Webchat::Table::MakeBlank($cgi);

	# Submit Button
	Webchat::Table::MakeSingle($cgi, $cgi->submit($chat{'TxtSubmitButton'}));

	# Bottom and end form
	Webchat::Table::MakeBottom($cgi);
	Webchat::Param::EmbedNormal($Database, $cgi, $param);
	print $cgi->end_form();


	# Unlink DBM with hash
	Webchat::Dbm::DBMClose(\%chat);
	Webchat::Dbm::DBMClose(\%realm);
}


#####################
# OptionSplit
#
# Splits options list into seperate elements of array

sub OptionSplit
{
	#####################
	# Data members

	my $array = shift;					# Reference to array
	my $data = shift;					# Retreived line


	#####################
	# Program area
	@{ $array } = split(/:/, $data);			# Split line
}


#####################
# Target Handler
#
# Will determine target to send the message privately to.  This will also create a
# parameter called SOURCE which will receive a copy as well.

sub TargetHandler
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Database statement
	my $param = shift;					# Parameter handle

	my $puppeteer;						# Puppeteer to send to


	#####################
	# Program area

	# Only do changes if target exists
	if (defined($param->{'TARGET'}))
	{
		# Unmangle names
		$param->{'TARGET'} = Webchat::Login::Unmangle($param->{'TARGET'});
		$param->{'USER'}   = Webchat::Login::Unmangle($param->{'USER'});

		# Retreive puppeteer name
		$param->{'OTARGET'} = $param->{'TARGET'};	# Keep original value
		($puppeteer) = Webchat::Database::DataGetPuppeteerLogin($Database, $param->{'TARGET'});

		# Assign values to parameters
		$param->{'TARGET'} = $puppeteer;
		$param->{'SOURCE'} = $param->{'USER'};
	}
}


